#!/bin/bash

if [ $# -lt 3 ] ; then
    echo Too few parameters!
    echo Usage: fairness_aggbw.sh start_iteration start_num_files sched
    echo "       [end_iteration (default: start_iteration)]"
    echo "       [end_num_files (default: start_num-files)] [skewness (default: 0)]"
    echo "       [T_wait (default: 4 ms)]"
    echo Skewness:
    echo "    0: all processes have the same weight"
    echo "    1: "
    echo "        2 procs: 2 1"
    echo "        3 procs: 2 1 1"
    echo "        4 procs: 2 2 1 1"
    echo "        5 procs: 2 2 1 1 1"
    echo "    2: "
    echo "        2 procs: 10 1"
    echo "        3 procs: 10 2 1"
    echo "        4 procs: 10 2 1 1"
    echo "        5 procs: 10 2 2 1 1"
    exit 1
fi

start_iter=$1
start_nfiles=$2
sched=$3
end_iter=${4:-$start_iter}
end_nfiles=${5:-$start_nfiles}
skewness=${6:-0}
T_wait=${7:-4}

if [ $sched != "anticipatory" ] && [ $sched != "bfq" ] && [ $sched != "yfq" ] && [ $sched != "scanedf" ] && [ $sched != "cfq" ] ; then
    echo Unaccepted scheduler type $sched
    exit
fi

if [ $skewness != 0 ] && [[ $sched == "cfq" || $sched == "anticipatory" ]] ; then
	echo Skewness value $skewness not allowed for $sched 
	exit
fi

echo Fairness and aggregate throughput test starting ...
printf "start_iter end_iter start_nfiles end_nfiles skew T_wait   sched\n"
printf "%9d %8d %12d %10d %4d %5d %10s\n\n" $start_iter $end_iter $start_nfiles\
    $end_nfiles $skewness $T_wait $sched

ver=fair_agg`date +%Y%m%d-%H%M`
bindir="/home/paolo/bin"

files[1]="/mnt/hdb20/1GB_file"
files[2]="/mnt/hdb5/1GB_file /mnt/hdb34/1GB_file"
files[3]="/mnt/hdb5/1GB_file /mnt/hdb20/1GB_file /mnt/hdb34/1GB_file"
files[4]="/mnt/hdb5/1GB_file /mnt/hdb12/1GB_file /mnt/hdb20/1GB_file \
/mnt/hdb34/1GB_file"
files[5]="/mnt/hdb5/1GB_file /mnt/hdb12/1GB_file /mnt/hdb20/1GB_file \
/mnt/hdb27/1GB_file /mnt/hdb34/1GB_file"

case $skewness in
      0) if [ $sched == "scanedf" ] ; then
		# where possible, we set higher weights for scanedf, in order 
		# to use shorter deadlines: see the ioprio->deadline 
		# conversion table in block/scanedf-iosched.c
		maxw=10 
		midw=10
		minw=10
	 else
		maxw=1
		midw=1
		minw=1
	 fi
	 ;;     
      1) if [ $sched == "scanedf" ] ; then
		maxw=4
		midw=4
		minw=2
	 else
		maxw=2
		midw=2
		minw=1
	 fi
	 ;;     
      2) maxw=10
	 midw=2
	 minw=1
	 ;;     
      *) echo Wrong skewness value $skewness; exit;;
esac
	
weights[1]="$minw"
weights[2]="$maxw $minw"
weights[3]="$maxw $midw $minw"
weights[4]="$maxw $midw $minw $minw"
weights[5]="$maxw $midw $midw $minw $minw"

dest_hd=hdb

# for offline testing
#alias mount="echo mount"
#alias umount="echo umount"

#measured in MB
#sizes="128 256 512 921"
sizes="256"
max_size=921

echo Creating needed files $f
# create files if needed
for f in ${files[5]} ; do
	mount `dirname $f`
	if [ \! -f $f ] ; then 
	    echo Preparing $f
	    dd if=/dev/zero of=$f bs=1M count=1024; 
	else
	    echo $f already exists
	fi
	umount `dirname $f`
done
echo Recall: the maximum file portion read is $max_size MB long

echo noop > /sys/block/$dest_hd/queue/scheduler
#rmmod $sched-iosched
modprobe $sched-iosched
echo echo "$sched > /sys/block/$dest_hd/queue/scheduler"
echo $sched > /sys/block/$dest_hd/queue/scheduler

case $sched in
    anticipatory)
	dest_param=antic_expire
	;;
    cfq | bfq)
	dest_param=slice_idle
	;;
    scanedf | yfq)
	dest_param=wait_expire
	;;
esac    

echo $T_wait > /sys/block/$dest_hd/queue/iosched/$dest_param
echo echo "$T_wait > /sys/block/$dest_hd/queue/iosched/$dest_param"
cat /sys/block/$dest_hd/queue/iosched/$dest_param
wait_tag=T_wait$T_wait

case $sched in
    bfq)
	start_budget=256
	max_budget=16384
	sysfs_par="max_budget"
	letter=b
    # echo 1 > /sys/block/$dest_hd/queue/iosched/max_dispatch
    # echo "echo 1 > /sys/block/$dest_hd/queue/iosched/max_dispatch"
    # cat /sys/block/$dest_hd/queue/iosched/max_dispatch
    ;;
    yfq)
	dmesg -n8
	# in number of requests
	start_budget=4
	max_budget=16
	sysfs_par="batch"
    	letter=b
	;;
    anticipatory)
    	echo 200000000 > /sys/block/$dest_hd/queue/iosched/read_expire
    	echo "echo 200000000 > /sys/block/$dest_hd/queue/iosched/read_expire"
    	cat /sys/block/$dest_hd/queue/iosched/read_expire
    	#echo 200000000 > /sys/block/$dest_hd/queue/iosched/read_batch_expire
    	#echo "echo 200000000 > /sys/block/$dest_hd/queue/iosched/read_batch_expire"
    	cat /sys/block/$dest_hd/queue/iosched/read_batch_expire
	
	# meaningless paramters
	start_budget=1
	max_budget=1
	sysfs_par=""
	letter=""
	;;
    cfq)
	start_budget=100000
	max_budget=100000
	#start_budget=100
	#max_budget=100
	sysfs_par="slice_sync"
	letter=s
	;;
    scanedf)
	# here budget means deadline granularity
	start_budget=20
	max_budget=640
	dmesg -n8
	sysfs_par="dline_granularity"
	letter=g
	;;
esac

echo Starting tests ...
# do the test
for ((iteration = $start_iter; iteration <= $end_iter; \
    iteration++)) ; do
    for ((nfiles = $start_nfiles; nfiles <= $end_nfiles; \
	nfiles++)) ; do
	rootdir="tmp_results/$sched/$wait_tag/skew$skewness/$ver/nfiles${nfiles}-iter$iteration/"
	mkdir -p "$rootdir/single_logs"
	for size in $sizes ; do
	    for ((budget = $start_budget; budget <= $max_budget; \
		budget *= 2)) ; do
		out=log-${size}M-$letter${budget}
		for f in  ${files[$nfiles]} ; do
		    mount `dirname $f`
		done
		if [ $sched == "bfq" ] || [ $sched == "yfq" ]  || \
			[ $sched == "scanedf" ] || [ $sched == "cfq"  ] ; then
			echo $budget > \
			    /sys/block/$dest_hd/queue/iosched/$sysfs_par
			echo -- File size = $size / $sysfs_par = $budget --
		else
		echo -- File size = $size --
		fi
		echo $out
		/usr/bin/time -f %e --output="$rootdir/delay_$out" \
		    sh read_files.sh "$rootdir/single_logs/thr_$out-file" \
		    "${files[$nfiles]}" $size "${weights[$nfiles]}"
		delay=`cat "$rootdir/delay_$out"`

		tot_size=0
		max_w=`echo ${weights[nfiles]} | awk "{print \\$1}"`
		for ((i = 1 ; i <= $nfiles ; i++)) ; do
		    w=`echo ${weights[nfiles]} | awk "{print $"$i"}"`
		    form="$tot_size + ($size.0 * $w) / $max_w"
		    tot_size=`printf "scale=8\n%s\n" "$form" | bc -l`
		done

		echo tot_size: $tot_size, delay: $delay
		bw_formula="(1.0 * $tot_size) / $delay"
		aggbw=`printf "scale=2\n%s\n" "$bw_formula" | bc -l`
		echo Aggregate throughput: $aggbw MB/sec
		echo $aggbw > "$rootdir/agg_thr_$out"
		echo ----------------------------------------------------------
		echo
		for f in ${files[$nfiles]} ; do
		    umount `dirname $f`
		done
	    done
	done
    done
done
