#!/bin/bash

# PARAMS: size budget nfiles sched
function start_noise() {
    size=$1
    budget=$2
    nfiles=$3
    sched=$4

    out=log-${size}M-b${budget}
    for f in ${files[$nfiles]} ; do
        umount `dirname $f`
    done
    for f in  ${files[$nfiles]} ; do
        mount `dirname $f`
    done
    sleep 13

    echo $budget > /sys/block/$dest_hd/queue/iosched/$sysfs_par
    echo -- File size = $size / Budget = $budget --
    echo $out
    /usr/bin/time -f %e --output="$rootdir/delay_$out" \
        sh read_files.sh "$rootdir/single_logs/thr_$out-file" \
        "${files[$nfiles]}" 
    delay=`cat "$rootdir/delay_$out"`

    echo noise duration: $delay
    echo ----------------------------------------------------------
    echo
}

function stop_noise() {
	killall -USR1 reader
}

# kill possible still alive readers
killall -9 reader

if [ $# -lt 3 ] ; then
    echo Too few parameters!
    echo Usage: vlc_test.sh start_iteration start_num_files sched

    echo "       [end_iteration (default: start_iteration)]"
    echo "       [end_num_files (default: start_num-files)]"
    exit 1
fi

start_iter=$1
start_nfiles=$2
sched=$3
end_iter=${4:-$start_iter}
end_nfiles=${5:-$start_nfiles}
skewness=${6:-0}

if [ $sched != "bfq" ] && [ $sched != "yfq" ] && [ $sched != "anticipatory" ] && [ $sched != "scanedf" ] && [ $sched != "cfq" ] ; then
    echo Unaccepted scheduler type $sched
    exit
fi

umask 2

echo Multimedia test starting ...
printf "start_iter end_iter start_nfiles end_nfiles sched\n"
printf "%9d %8d %12d %10d %5s\n\n" $start_iter $end_iter $start_nfiles \
    $end_nfiles $sched

ver=fair_agg`date +%Y%m%d-%H%M`
bindir="/home/paolo/bin"

files[1]="/mnt/hdb20/1GB_file"
files[2]="/mnt/hdb5/1GB_file /mnt/hdb34/1GB_file"
files[3]="/mnt/hdb5/1GB_file /mnt/hdb20/1GB_file /mnt/hdb34/1GB_file"
files[4]="/mnt/hdb5/1GB_file /mnt/hdb12/1GB_file /mnt/hdb20/1GB_file \
/mnt/hdb34/1GB_file"
files[5]="/mnt/hdb5/1GB_file /mnt/hdb12/1GB_file /mnt/hdb20/1GB_file \
/mnt/hdb27/1GB_file /mnt/hdb34/1GB_file"

dest_hd=hdb

# for offline testing
#alias mount="echo mount"
#alias umount="echo umount"

#measured in MB
sizes="921"
max_size=921

echo Creating needed files $f
# create files if needed
for f in ${files[5]} ; do
	mount `dirname $f`
	if ! [ -f $f ] ; then 
	    echo Preparing $f
	    dd if=/dev/zero of=$f bs=1M count=$max_size; 
	else
	    echo $f already exists
	fi
	umount `dirname $f`
done
echo Recall: files are currently $max_size MB long

echo noop > /sys/block/$dest_hd/queue/scheduler
#rmmod $sched-iosched
modprobe $sched-iosched
echo echo "$sched > /sys/block/$dest_hd/queue/scheduler"
echo $sched > /sys/block/$dest_hd/queue/scheduler

if [ $sched == "bfq" ] ; then
    # in sectors
    # XXX start_budget=512
    start_budget=4096
    max_budget=16384
    sysfs_par="max_budget"
elif [ $sched == "yfq" ] ; then
    # in number of requests
    start_budget=4
    max_budget=$start_budget
    sysfs_par="batch"
elif [ $sched == "cfq" ] ; then
    # in ms
    start_budget=20
    max_budget=$start_budget
    sysfs_par="slice_sync"
elif [ $sched == "scanedf" ] ; then
    # in ms
    start_budget=20
    max_budget=$start_budget
    sysfs_par="dline_granularity"
elif [ $sched == "anticipatory" ] ; then
    echo 20000000 > /sys/block/$dest_hd/queue/iosched/read_expire
    start_budget=8
    max_budget=8
    sysfs_par="antic_expire"
else
    echo "Invalid scheduler name $sched"
    exit
fi

echo Starting tests ...
# do the test
for ((iteration = $start_iter; iteration <= $end_iter; \
    iteration++)) ; do
    for ((nfiles = $start_nfiles; nfiles <= $end_nfiles; \
	nfiles++)) ; do
	rootdir="video_results/$sched/T_wait4/skew$skewness/$ver/nfiles${nfiles}-iter$iteration/"
	mkdir -p "$rootdir/single_logs"
	for size in $sizes ; do
	    for ((budget = $start_budget; budget <= $max_budget; \
		budget *= 2)) ; do

		out=log-${size}M-b${budget}
		start_noise $size $budget $nfiles $sched &
		iostat -tmd /dev/hdb 5 > $rootdir/${out}_iostat &
		su vlc -c "sh vlc_auto.sh /tmp/videos"
		killall iostat
		mv /tmp/videos $rootdir/${out}_videos
		stop_noise
		sleep 4
		if ((max_budget==0)) ; then
		    break
		fi
	    done
	done
    done
done
